# Generated by roxygen2: do not edit by hand

S3method(ctl_new_pillar,one_skim_df)
S3method(format,summary_skim_df)
S3method(get_skimmers,AsIs)
S3method(get_skimmers,Date)
S3method(get_skimmers,POSIXct)
S3method(get_skimmers,Timespan)
S3method(get_skimmers,character)
S3method(get_skimmers,complex)
S3method(get_skimmers,default)
S3method(get_skimmers,difftime)
S3method(get_skimmers,factor)
S3method(get_skimmers,haven_labelled)
S3method(get_skimmers,list)
S3method(get_skimmers,logical)
S3method(get_skimmers,numeric)
S3method(get_skimmers,ts)
S3method(knit_print,one_skim_df)
S3method(knit_print,skim_df)
S3method(knit_print,skim_list)
S3method(knit_print,summary_skim_df)
S3method(mutate,skim_df)
S3method(print,skim_df)
S3method(print,skim_list)
S3method(print,summary_skim_df)
S3method(repr_text,one_skim_df)
S3method(repr_text,skim_df)
S3method(repr_text,skim_list)
S3method(skim_by_type,data.frame)
S3method(skim_by_type,data.table)
S3method(skim_by_type,grouped_df)
S3method(summary,skim_df)
S3method(tbl_format_header,one_skim_df)
S3method(to_long,default)
S3method(to_long,skim_df)
S3method(vec_cast,skim_df.skim_df)
S3method(vec_cast,skim_df.tbl_df)
S3method(vec_cast,tbl_df.skim_df)
S3method(vec_ptype2,skim_df.skim_df)
S3method(vec_ptype2,skim_df.tbl_df)
S3method(vec_ptype2,tbl_df.skim_df)
export("%>%")
export(assert_is_one_skim_df)
export(assert_is_skim_df)
export(assert_is_skim_list)
export(base_skimmers)
export(bind)
export(complete_rate)
export(contains)
export(could_be_skim_df)
export(data_cols)
export(data_rows)
export(df_name)
export(dt_key)
export(ends_with)
export(everything)
export(fix_windows_histograms)
export(focus)
export(get_default_skimmer_names)
export(get_default_skimmers)
export(get_one_default_skimmer)
export(get_one_default_skimmer_names)
export(get_sfl)
export(get_skimmers)
export(group_names)
export(has_skim_type_attribute)
export(has_skimmers)
export(has_skimr_attributes)
export(has_type_column)
export(has_variable_column)
export(inline_hist)
export(inline_linegraph)
export(is_data_frame)
export(is_one_skim_df)
export(is_skim_df)
export(is_skim_list)
export(list_lengths_max)
export(list_lengths_median)
export(list_lengths_min)
export(list_max_length)
export(list_min_length)
export(matches)
export(max_char)
export(min_char)
export(modify_default_skimmers)
export(n_complete)
export(n_empty)
export(n_missing)
export(n_unique)
export(n_whitespace)
export(num_range)
export(one_of)
export(partition)
export(sfl)
export(skim)
export(skim_format)
export(skim_tee)
export(skim_to_list)
export(skim_to_wide)
export(skim_with)
export(skim_without_charts)
export(skimmers_used)
export(sorted_count)
export(starts_with)
export(to_long)
export(top_counts)
export(ts_end)
export(ts_start)
export(yank)
importFrom(dplyr,mutate)
importFrom(knitr,knit_print)
importFrom(magrittr,"%>%")
importFrom(pillar,ctl_new_pillar)
importFrom(pillar,tbl_format_header)
importFrom(repr,repr_text)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(tidyselect,contains)
importFrom(tidyselect,ends_with)
importFrom(tidyselect,everything)
importFrom(tidyselect,matches)
importFrom(tidyselect,num_range)
importFrom(tidyselect,one_of)
importFrom(tidyselect,starts_with)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_ptype2)
