# Extracted from test-skim_print.R:15

# setup ------------------------------------------------------------------------
library(testthat)
test_env <- simulate_test_env(package = "skimr", path = "..")
attach(test_env, warn.conflicts = FALSE)

# test -------------------------------------------------------------------------
skip_if_not(l10n_info()$`UTF-8`)
withr::local_options(list(cli.unicode = FALSE))
expect_snapshot( skim(dplyr::group_by(iris, Species)))
