---
title: "Skimr defaults"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    latex_engine: xelatex
vignette: >
  %\VignetteIndexEntry{Skimr defaults}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Introduction

This vignette simply displays the default data types and summary functions for
`skimr`. Customizing `skimr` is explained in the **Using Skimr** vignette.

## The base skimmers

`skimr` has a group of functions that it applies to all data types. We call
these the "base"" skimmers:

*  `n_missing`: The number of missing values in the column.
*  `complete_rate`: The ratio of non-missing values to the total values in the
   column.

## Default skimmers

To learn more about the functions used in this package, use the function
`get_default_skimmer_names()`.

```{r}
library(skimr)
get_default_skimmer_names()
```

The counterpart to this function is `get_default_skimmers()`, which returns the
functions themselves. If you are interested in a particular class within
`skimr`, pass it as a string to either function.

```{r}
get_default_skimmer_names("numeric")
```

The same information is stored in the `skimmers_used` attribute of the object
returned by `skim()`.
