---
output:
  html_document: default
  pdf_document:
    latex_engine: xelatex
---

```{r}
library(skimr)
data <- read.csv("https://github.com/ropensci/skimr/files/11214724/githubdata.csv")
```

```{r}
skim(data)
```
```{r}
hist(data$h, breaks =12)
```

