# The summary print method prints the correct object

    Code
      skim_summary_input
    Output
      -- Data Summary ------------------------
                                 Values
      Name                       iris  
      Number of rows             150   
      Number of columns          5     
      _______________________          
      Column type frequency:           
        factor                   1     
        numeric                  4     
      ________________________         
      Group variables            None  

# The summary print method prints the correct object when piped

    Code
      summary_input
    Output
      -- Data Summary ------------------------
                                 Values
      Name                       iris  
      Number of rows             150   
      Number of columns          5     
      _______________________          
      Column type frequency:           
        factor                   1     
        numeric                  4     
      ________________________         
      Group variables            None  

